"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.run = void 0;
const tslib_1 = require("tslib");
const path_1 = require("path");
const architect_1 = require("@angular-devkit/architect");
const core_1 = require("@angular-devkit/core");
const build_webpack_1 = require("@angular-devkit/build-webpack");
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
const project_graph_1 = require("@nrwl/workspace/src/core/project-graph");
const normalize_1 = require("@nrwl/node/src/utils/normalize");
const node_config_1 = require("@nrwl/node/src/utils/node.config");
const buildable_libs_utils_1 = require("@nrwl/workspace/src/utils/buildable-libs-utils");
const config_1 = require("@nrwl/node/src/utils/config");
const ngcli_adapter_1 = require("@nrwl/tao/src/commands/ngcli-adapter");
const generate_package_json_1 = require("../../utils/generate-package-json");
exports.default = architect_1.createBuilder(run);
function run(options, context) {
    const projGraph = project_graph_1.createProjectGraph();
    if (!options.buildLibsFromSource) {
        const { target, dependencies } = buildable_libs_utils_1.calculateProjectDependencies(projGraph, context);
        options.tsConfig = buildable_libs_utils_1.createTmpTsConfig(options.tsConfig, context.workspaceRoot, target.data.root, dependencies);
        if (!buildable_libs_utils_1.checkDependentProjectsHaveBeenBuilt(context, dependencies)) {
            return { success: false };
        }
    }
    let normalizeOptions;
    return rxjs_1.from(getRoots(context))
        .pipe(operators_1.map(({ sourceRoot, projectRoot }) => {
        normalizeOptions = normalize_1.normalizeBuildOptions(options, context.workspaceRoot, sourceRoot, projectRoot);
        return normalizeOptions;
    }), operators_1.map((options) => {
        let config = node_config_1.getNodeWebpackConfig(options);
        if (options.webpackConfig) {
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-ignore
            // eslint-disable-next-line @typescript-eslint/no-var-requires
            config = require(options.webpackConfig)(config, {
                options,
                configuration: context.target.configuration
            });
        }
        return config;
    }), operators_1.concatMap((config) => build_webpack_1.runWebpack(config, context, {
        logging: (stats) => {
            context.logger.info(stats.toString(config.stats));
        },
        webpackFactory: require('webpack')
    })), operators_1.map((buildEvent) => {
        buildEvent.outfile = path_1.resolve(context.workspaceRoot, options.outputPath, config_1.OUT_FILENAME);
        return buildEvent;
    }), operators_1.tap(() => generate_package_json_1.generatePackageJson(context.target.project, normalizeOptions, context.workspaceRoot)));
}
exports.run = run;
function getRoots(context) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const workspaceHost = core_1.workspaces.createWorkspaceHost(new ngcli_adapter_1.NxScopedHost(core_1.normalize(context.workspaceRoot)));
        const { workspace } = yield core_1.workspaces.readWorkspace('', workspaceHost);
        const { project } = context.target;
        const { sourceRoot, root } = workspace.projects.get(project);
        if (sourceRoot && root) {
            return { sourceRoot, projectRoot: root };
        }
        context.reportStatus('Error');
        const message = `${project} does not have a sourceRoot or root. Please define both.`;
        context.logger.error(message);
        throw new Error(message);
    });
}
//# sourceMappingURL=build.impl.js.map